@@ -3,6 +3,7 @@ from collections import deque, Counter, OrderedDict
 from contextlib import contextmanager
 import json, time, random, logging
 from sqlite3.dbapi2 import ProgrammingError
+from pathlib import Path
 from json import encoder
 import sqlite3
 from types import FunctionType
@@ -33,6 +34,7 @@ class EasySqliteTokenError(Exception):
 #Columns = Excluding[str, Sequence[str]] # If Excluding existed
 Params = Union[List[Any], Tuple[Any, ...]]
 Columns = Union[List[str], Tuple[str, ...]]
+FilePath = Union[str, Path]
 
 
 class SqliteDB:
@@ -42,7 +44,7 @@ class SqliteDB:
     def __repr__(self):
         return custom_repr(self, 'file')
 
-    def __init__(self, file=':memory:'):
+    def __init__(self, file: FilePath):
         self.file = file
 
     def new_connection(self):
@@ -113,7 +115,7 @@ class SqliteDB:
 
 class SqliteTable:
 
-    def __init__(self, file: str, table_name: str):
+    def __init__(self, file: FilePath, table_name: str):
         self.file = file
         self.table_name = table_name
         self.db = SqliteDB(self.file)
@@ -141,6 +143,17 @@ class SqliteTable:
         params = [*kwargs.values()]
         return self.db._execute(query, params)
 
+    def insert_row_or_ignore(self, **kwargs):
+        assert kwargs, 'Nothing to set'
+        table = self.table_name
+        columns = ', '.join(kwargs.keys())
+        marks = ', '.join('?' for _ in kwargs)
+        query = f'INSERT OR IGNORE INTO {table} ({columns}) VALUES ({marks})'
+        params = [*kwargs.values()]
+        cursor = self.db._execute(query, params)
+        did_insert = (cursor.rowcount != 0)
+        return did_insert
+
     def _make_query(self, columns: Columns = None, where: str = None):
         table = self.table_name
         what = self._make_columns(columns)
@@ -204,7 +217,8 @@ class IndexedSqliteTable(SqliteTable):
 
     _repr_keys = ['file', 'table_name', 'index_columns']
 
-    def __init__(self, file: str, table_name: str, index_columns: List[str]):
+    def __init__(self, file: FilePath, table_name: str,
+                 index_columns: List[str]):
         super().__init__(file, table_name)
         self.index_columns = index_columns
 
@@ -263,7 +277,7 @@ class IndexedSqliteTable(SqliteTable):
 
 class SqliteStore(SqliteTable):
 
-    def __init__(self, file: str, table_name: str):
+    def __init__(self, file: FilePath, table_name: str):
         super().__init__(file, table_name)
         self.db.execute(f'''
         CREATE TABLE IF NOT EXISTS {self.table_name}(
@@ -324,12 +338,12 @@ class SqliteStore(SqliteTable):
         Requires a token provided by the exclusive access context
         manager self.wait_token() or self.ask_token().
         '''
-        with self.assert_token(key, token=token, _warn_del=False):
+        with self.assert_token(key, token=token):
             self.__del_assuming_token(key)
         return
 
     def ask_del(self, key: str):
-        with self.ask_token(key, _warn_del=False) as token:
+        with self.ask_token(key) as token:
             if not token:
                 return False
             self.__del_assuming_token(key)
@@ -345,7 +359,7 @@ class SqliteStore(SqliteTable):
     def wait_del(self, key: str, request_every: float = 0.02,
                  timeout: Optional[float] = 3):
         wait = self.wait_token(key, request_every=request_every,
-                               timeout=timeout, _warn_del=False)
+                               timeout=timeout)
         with wait:
             self.__del_assuming_token(key)
 
@@ -357,7 +371,7 @@ class SqliteStore(SqliteTable):
             self.set(key, value, token)
 
     @contextmanager
-    def ask_token(self, *keys: str, max_duration: float = 0.5, _warn_del=True):
+    def ask_token(self, *keys: str, max_duration: float = 0.5):
         '''
         with table.ask_token('some_key') as token:
             if not token:
@@ -376,11 +390,11 @@ class SqliteStore(SqliteTable):
             yield token if gained_access else None
         finally:
             for key in keys:
-                self._unlock(key, token, max_duration, _warn_del)
+                self._unlock(key, token, max_duration)
         return
 
     @contextmanager
-    def assert_token(self, *keys: str, token: float, _warn_del=True):
+    def assert_token(self, *keys: str, token: float):
         assert keys, 'You must specify keys to be locked explicitely'
         max_duration = 0.5
         try:
@@ -391,7 +405,7 @@ class SqliteStore(SqliteTable):
             yield
         finally:
             for key in keys:
-                self._unlock(key, token, max_duration, _warn_del)
+                self._unlock(key, token, max_duration)
         return
 
     @contextmanager
@@ -421,7 +435,7 @@ class SqliteStore(SqliteTable):
             yield token
         finally:
             for key in keys:
-                self._unlock(key, token, max_duration, _warn_del)
+                self._unlock(key, token, max_duration)
         return
 
     def _ask_access(self, key: str, token: float, max_duration: float):
@@ -443,26 +457,16 @@ class SqliteStore(SqliteTable):
         if cursor.rowcount > 0:  # Race winner
             return True
         # Maybe the key was not even present:
-        cursor = self.db._execute(
-            f'''
-        INSERT OR IGNORE
-        INTO
-            {self.table_name} (key, lock_token, locked_until)
-        VALUES
-            (?,?,?)
-        ''', [key, token, until])
-        if cursor.rowcount > 0:
-            return True
-        return False
+        return self.insert_row_or_ignore(
+            key=key,
+            lock_token=token,
+            locked_until=until,
+        )
 
-    def _unlock(self, key: str, token: float, max_duration: float,
-                _warn_del: bool):
+    def _unlock(self, key: str, token: float, max_duration: float):
         d = self._current_lock(key)
         if not d:
-            if _warn_del:
-                logging.warning(
-                    f'Key {repr(key)} was deleted by another thread or process during exclusive access'
-                )
+            # Entry was deleted (and unlocked)
             return
         remaining = d['locked_until'] - time.time()
         if remaining < 0:
